/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup ble_sdk_uart_over_ble_main main.c
 * @{
 * @ingroup  ble_sdk_app_nus_eval
 * @brief    UART over BLE application main file.
 *
 * This file contains the source code for a sample application that uses the Nordic UART service.
 * This application uses the @ref srvlib_conn_params module.
 */


#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"
#include "bsp.h" //BRN
#include <nrf_delay.h>
#include "app_pwm.h"
#include <nrfx_timer.h>

#include "nrf_drv_pwm.h"

#include "ble_srv_common.h"
#include "nrf_dfu_ble_svci_bond_sharing.h"
#include "nrf_svci_async_function.h"
#include "nrf_svci_async_handler.h"
#include "ble_dfu.h"
#include "peer_manager.h"
#include "fds.h"
#include "ble_conn_state.h"
#include "ble.h"
#include "nrf_bootloader_info.h"
#include "nrf_drv_clock.h"
#include "nrf_power.h"

#include "nrf_sdh.h"
#include "nrf_sdh_ble.h"
#include "nrf_fstorage_sd.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "nrf_drv_saadc.h"

#define APP_BLE_CONN_CFG_TAG            1                                           /**< A tag identifying the SoftDevice BLE configuration. */

#define DEVICE_NAME                     "V015"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_BLE_OBSERVER_PRIO           3                                           /**< Application's BLE observer priority. You shouldn't need to modify this value. */

#define APP_ADV_INTERVAL                64                                          /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */

#define APP_ADV_DURATION                6000//18000                                       /**< The advertising duration (180 seconds) in units of 10 milliseconds. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */

#define TX_POWER_LEVEL                  (0)   
//BRN
#define STATIC_PASSKEY "123456"
static ble_opt_t m_static_pin_option;

//BRN

#define  SEC_PARAM_BOND  1
/**< Perform bonding. */
#define  SEC_PARAM_MITM  1
/**< Man In The Middle protection not required. */
#define  SEC_PARAM_LESC  0
/**< LE Secure Connections not enabled. */
#define  SEC_PARAM_KEYPRESS  0
/**< Keypress notifications not enabled. */
#define  SEC_PARAM_IO_CAPABILITIES   BLE_GAP_IO_CAPS_DISPLAY_ONLY //BLE_GAP_IO_CAPS_NONE
/**< No I/O capabilities. */
#define  SEC_PARAM_OOB  0
/**< Out Of Band data not available. */
#define  SEC_PARAM_MIN_KEY_SIZE  7
/**< Minimum encryption key size. */
#define  SEC_PARAM_MAX_KEY_SIZE  16
/**< Maximum encryption key size. */


#define BEEP_PIN 14
#define MOTOR_PIN 7
#define LED_PIN 12
#define KEY_PIN 11
#define YP_PIN 29
#define USB_DET_PIN 8
#define HALL_PWR_PIN 31
#define HALL_OUT_PIN 6
#define BAT_VOL_PIN 30

#define LED_ST_OFF 0
#define LED_ST_SLOW_BLINK 1
#define LED_ST_ON 2
#define LED_ST_FAST_BLINK 3

#define INIT_BAT_PWM 200
#define INIT_USB_PWM 200

#define AUTO_OFF_TIME 900


BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);                                   /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                             /**< Context for the Queued Write module.*/
BLE_ADVERTISING_DEF(m_advertising);                                                 /**< Advertising module instance. */
//APP_TIMER_DEF(m_app_10ms_tmr); //BRN
APP_TIMER_DEF(m_app_1ms_tmr); //BRN




static volatile bool ready_flag;            // A flag indicating PWM status.

void pwm_ready_callback(uint32_t pwm_id)    // PWM callback function
{
    ready_flag = true;
}

static nrf_drv_pwm_t m_pwm0 = NRF_DRV_PWM_INSTANCE(0);

APP_PWM_INSTANCE(PWM1,1);                   // Create the instance "PWM1" using TIMER1.

static uint16_t   m_conn_handle          = BLE_CONN_HANDLE_INVALID;                 /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
static ble_uuid_t m_adv_uuids[]          =                                          /**< Universally unique service identifier. */
{
    {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};




uint8_t rcvData[8]; //BRN
uint8_t sendData[8];//BRN
uint16_t DataLength; //BRN
uint8_t testData = 0;//BRN
uint32_t brnEc;
bool writeFlashCmd = 0;
bool readFlashCmd = 0;
bool advStartedFlag = 0;
bool connectedFlag = 0; 
bool newBleDataFlag = 0;
uint16_t updateDevStateCnt;
const uint8_t firmwareVer[2] = {0x01,0x03};
bool decanterStateFlag = 0;
bool beepOnFlag = 0;
bool systemOnFlag = 0;
bool preKeyState = 1;
bool ypSwitchState = 0;
bool aerateStartFlag = 0;
bool aerateDoneFlag = 0;
bool motorOnFlag = 0;
bool uploadYpState = 0;
uint8_t keyDebounceCnt = 0;
uint16_t ledBlinkingCnt = 0;
uint16_t beepTimeCnt = 0;
uint8_t beepTimes = 0;
uint16_t ypSwitchDetCnt = 0;
uint32_t brnEc = 0;
uint16_t enterSleepDelay = 0;
uint16_t sysAutoOffCnt = 0;
uint16_t msCnt = 0;
uint16_t aerateTimer = 0;
uint8_t ledState = 0;
uint16_t beepIntervalCnt = 0;
uint8_t  aerateDoneBeepCnt = 0;
uint8_t uploadYpStateDelay = 0;
nrf_saadc_value_t  saadc_val;
uint16_t adcReadDelay = 0;
uint8_t batLowCnt = 0;
bool batLowFlag = 0;
bool preHallSnrLvl = 0;
bool hallStUpdateFlag = 0;
uint8_t hallSnrDebounceCnt = 0;
uint16_t rotateSpeedTimeCnt = 0;
uint16_t rotateSpeedTime = 0;
uint16_t stuckCnt = 0;
bool stuckedFlag = 0;
uint8_t pwmSetting = 0;
uint8_t pwmSettingBackup = 0;
bool usbConnectedFlag = 0;
uint8_t usbConnectCnt = 0;
uint16_t motorOffDelayCnt = 0;
bool motorDelayStopFlag = 0;
bool motorDelayOffTimeSetFlag = 0;
bool backToFixedPosFlag;
uint8_t hallBlockCnt = 0;
uint16_t motorDelayOffTimeSet;



static nrf_pwm_values_individual_t m_demo1_seq_values;
static nrf_pwm_sequence_t const    m_demo1_seq =
{
    .values.p_individual = &m_demo1_seq_values,
    .length              = NRF_PWM_VALUES_LENGTH(m_demo1_seq_values),
    .repeats             = 0,
    .end_delay           = 0
};

static void demo1_handler(nrf_drv_pwm_evt_type_t event_type)
{
   
}
static void pwm_drv_init(void)
{
    
    nrf_drv_pwm_config_t const config0 =
    {
        .output_pins =
        {
					MOTOR_PIN,
          NRF_DRV_PWM_PIN_NOT_USED,
          NRF_DRV_PWM_PIN_NOT_USED,
          NRF_DRV_PWM_PIN_NOT_USED	            
        },
        .irq_priority = APP_IRQ_PRIORITY_LOWEST,
        .base_clock   = NRF_PWM_CLK_4MHz,
        .count_mode   = NRF_PWM_MODE_UP,
        .top_value    = 255,
        .load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
        .step_mode    = NRF_PWM_STEP_AUTO
    };
    APP_ERROR_CHECK(nrf_drv_pwm_init(&m_pwm0, &config0, NULL/*demo1_handler*/));


    m_demo1_seq_values.channel_0 = 255;


    (void)nrf_drv_pwm_simple_playback(&m_pwm0, &m_demo1_seq, 1,
                                      NRF_DRV_PWM_FLAG_LOOP);
}





static void advertising_init(void);

static void advertising_start(bool erase_bonds);      

static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt);


NRF_FSTORAGE_DEF(nrf_fstorage_t fstorage) =
{
    /* Set a handler for fstorage events. */
    .evt_handler = fstorage_evt_handler,

    /* These below are the boundaries of the flash space assigned to this instance of fstorage.
     * You must set these manually, even at runtime, before nrf_fstorage_init() is called.
     * The function nrf5_flash_end_addr_get() can be used to retrieve the last address on the
     * last page of flash available to write data. */
    .start_addr = 0x77000,
    .end_addr   = 0x77FFF,
};

/* Dummy data to write to flash. */
//static uint32_t  pairKey;

static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt)
{
    if (p_evt->result != NRF_SUCCESS)
    {
        NRF_LOG_INFO("--> Event received: ERROR while executing an fstorage operation.");
        return;
    }

    switch (p_evt->id)
    {
        case NRF_FSTORAGE_EVT_WRITE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: wrote %d bytes at address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        case NRF_FSTORAGE_EVT_ERASE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: erased %d page from address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        default:
            break;
    }
}



//static uint32_t nrf5_flash_end_addr_get()
//{
//    uint32_t const bootloader_addr = NRF_UICR->NRFFW[0];
//    uint32_t const page_sz         = NRF_FICR->CODEPAGESIZE;
//    uint32_t const code_sz         = NRF_FICR->CODESIZE;

//    return (bootloader_addr != 0xFFFFFFFF ?
//            bootloader_addr : (code_sz * page_sz));
//}

void wait_for_flash_ready(nrf_fstorage_t const * p_fstorage)
{
    /* While fstorage is busy, sleep and wait for an event. */
    while (nrf_fstorage_is_busy(p_fstorage))
    {
       sd_app_evt_wait();
    }
}



/**@brief Handler for shutdown preparation.
 *
 * @details During shutdown procedures, this function will be called at a 1 second interval
 *          untill the function returns true. When the function returns true, it means that the
 *          app is ready to reset to DFU mode.
 *
 * @param[in]   event   Power manager event.
 *
 * @retval  True if shutdown is allowed by this power manager handler, otherwise false.
 */
static bool app_shutdown_handler(nrf_pwr_mgmt_evt_t event)
{
    switch (event)
    {
        case NRF_PWR_MGMT_EVT_PREPARE_DFU:
            NRF_LOG_INFO("Power management wants to reset to DFU mode.");
            // YOUR_JOB: Get ready to reset into DFU mode
            //
            // If you aren't finished with any ongoing tasks, return "false" to
            // signal to the system that reset is impossible at this stage.
            //
            // Here is an example using a variable to delay resetting the device.
            //
            // if (!m_ready_for_reset)
            // {
            //      return false;
            // }
            // else
            //{
            //
            //    // Device ready to enter
            //    uint32_t err_code;
            //    err_code = sd_softdevice_disable();
            //    APP_ERROR_CHECK(err_code);
            //    err_code = app_timer_stop_all();
            //    APP_ERROR_CHECK(err_code);
            //}
            break;

        default:
            // YOUR_JOB: Implement any of the other events available from the power management module:
            //      -NRF_PWR_MGMT_EVT_PREPARE_SYSOFF
            //      -NRF_PWR_MGMT_EVT_PREPARE_WAKEUP
            //      -NRF_PWR_MGMT_EVT_PREPARE_RESET
            return true;
    }

    NRF_LOG_INFO("Power management allowed to reset to DFU mode.");
    return true;
}

//lint -esym(528, m_app_shutdown_handler)
/**@brief Register application shutdown handler with priority 0.
 */
NRF_PWR_MGMT_HANDLER_REGISTER(app_shutdown_handler, 0);


static void buttonless_dfu_sdh_state_observer(nrf_sdh_state_evt_t state, void * p_context)
{
    if (state == NRF_SDH_EVT_STATE_DISABLED)
    {
        // Softdevice was disabled before going into reset. Inform bootloader to skip CRC on next boot.
        nrf_power_gpregret2_set(BOOTLOADER_DFU_SKIP_CRC);

        //Go to system off.
        nrf_pwr_mgmt_shutdown(NRF_PWR_MGMT_SHUTDOWN_GOTO_SYSOFF);
    }
}

/* nrf_sdh state observer. */
NRF_SDH_STATE_OBSERVER(m_buttonless_dfu_state_obs, 0) =
{
    .handler = buttonless_dfu_sdh_state_observer,
};



// YOUR_JOB: Update this code if you want to do anything given a DFU event (optional).
/**@brief Function for handling dfu events from the Buttonless Secure DFU service
 *
 * @param[in]   event   Event from the Buttonless Secure DFU service.
 */
static void ble_dfu_evt_handler(ble_dfu_buttonless_evt_type_t event)
{
    switch (event)
    {
        case BLE_DFU_EVT_BOOTLOADER_ENTER_PREPARE:
            NRF_LOG_INFO("Device is preparing to enter bootloader mode.");
            // YOUR_JOB: Disconnect all bonded devices that currently are connected.
            //           This is required to receive a service changed indication
            //           on bootup after a successful (or aborted) Device Firmware Update.
            break;

        case BLE_DFU_EVT_BOOTLOADER_ENTER:
            // YOUR_JOB: Write app-specific unwritten data to FLASH, control finalization of this
            //           by delaying reset by reporting false in app_shutdown_handler
            NRF_LOG_INFO("Device will enter bootloader mode.");
            break;

        case BLE_DFU_EVT_BOOTLOADER_ENTER_FAILED:
            NRF_LOG_ERROR("Request to enter bootloader mode failed asynchroneously.");
            // YOUR_JOB: Take corrective measures to resolve the issue
            //           like calling APP_ERROR_CHECK to reset the device.
            break;

        case BLE_DFU_EVT_RESPONSE_SEND_ERROR:
            NRF_LOG_ERROR("Request to send a response to client failed.");
            // YOUR_JOB: Take corrective measures to resolve the issue
            //           like calling APP_ERROR_CHECK to reset the device.
            APP_ERROR_CHECK(false);
            break;

        default:
            NRF_LOG_ERROR("Unknown event from ble_dfu_buttonless.");
            break;
    }
}



/**@brief Function for handling Peer Manager events.
 *
 * @param[in] p_evt  Peer Manager event.
 */
static void pm_evt_handler(pm_evt_t const * p_evt)
{
    ret_code_t err_code;

    switch (p_evt->evt_id)
    {
        case PM_EVT_BONDED_PEER_CONNECTED:
        {
            NRF_LOG_INFO("Connected to a previously bonded device.");
        } break;

        case PM_EVT_CONN_SEC_SUCCEEDED:
        {
            NRF_LOG_INFO("Connection secured: role: %d, conn_handle: 0x%x, procedure: %d.",
                         ble_conn_state_role(p_evt->conn_handle),
                         p_evt->conn_handle,
                         p_evt->params.conn_sec_succeeded.procedure);
        } break;

        case PM_EVT_CONN_SEC_FAILED:
        {
            /* Often, when securing fails, it shouldn't be restarted, for security reasons.
             * Other times, it can be restarted directly.
             * Sometimes it can be restarted, but only after changing some Security Parameters.
             * Sometimes, it cannot be restarted until the link is disconnected and reconnected.
             * Sometimes it is impossible, to secure the link, or the peer device does not support it.
             * How to handle this error is highly application dependent. */
        } break;

        case PM_EVT_CONN_SEC_CONFIG_REQ:
        {
            // Reject pairing request from an already bonded peer.
            pm_conn_sec_config_t conn_sec_config = {.allow_repairing = 1};
            pm_conn_sec_config_reply(p_evt->conn_handle, &conn_sec_config);
        } break;

        case PM_EVT_STORAGE_FULL:
        {
            // Run garbage collection on the flash.
            err_code = fds_gc();
            if (err_code == FDS_ERR_NO_SPACE_IN_QUEUES)
            {
                // Retry.
            }
            else
            {
                APP_ERROR_CHECK(err_code);
            }
        } break;

        case PM_EVT_PEERS_DELETE_SUCCEEDED:
        {
            advertising_start(false);
					  //advStartedFlag = 1;
        } break;

        case PM_EVT_PEER_DATA_UPDATE_FAILED:
        {
            // Assert.
            APP_ERROR_CHECK(p_evt->params.peer_data_update_failed.error);
        } break;

        case PM_EVT_PEER_DELETE_FAILED:
        {
            // Assert.
            APP_ERROR_CHECK(p_evt->params.peer_delete_failed.error);
        } break;

        case PM_EVT_PEERS_DELETE_FAILED:
        {
            // Assert.
            APP_ERROR_CHECK(p_evt->params.peers_delete_failed_evt.error);
        } break;

        case PM_EVT_ERROR_UNEXPECTED:
        {
            // Assert.
            APP_ERROR_CHECK(p_evt->params.error_unexpected.error);
        } break;

        case PM_EVT_CONN_SEC_START:
        case PM_EVT_PEER_DATA_UPDATE_SUCCEEDED:
        case PM_EVT_PEER_DELETE_SUCCEEDED:
        case PM_EVT_LOCAL_DB_CACHE_APPLIED:
        case PM_EVT_LOCAL_DB_CACHE_APPLY_FAILED:
            // This can happen when the local DB has changed.
        case PM_EVT_SERVICE_CHANGED_IND_SENT:
        case PM_EVT_SERVICE_CHANGED_IND_CONFIRMED:
        default:
            break;
    }
}



/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

/**@brief Function for initializing the timer module.
 */
static void timers_init(void)
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
																					
//BRN
		uint8_t passkey[] = STATIC_PASSKEY;                                                     
   m_static_pin_option.gap_opt.passkey.p_passkey = passkey;
   err_code =  sd_ble_opt_set(BLE_GAP_OPT_PASSKEY, &m_static_pin_option);
   APP_ERROR_CHECK(err_code); 
//BRN																							
																					
}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_evt       Nordic UART Service event.
 */
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_evt_t * p_evt)
{

    if (p_evt->type == BLE_NUS_EVT_RX_DATA)
    {
        //uint32_t err_code;

        NRF_LOG_DEBUG("Received data from BLE NUS. Writing data on UART.");
        NRF_LOG_HEXDUMP_DEBUG(p_evt->params.rx_data.p_data, p_evt->params.rx_data.length);
        //bsp_board_led_invert(BSP_BOARD_LED_3); //BRN
        if(p_evt->params.rx_data.length == 4)
				{
					for (uint32_t i = 0; i < p_evt->params.rx_data.length; i++)
          {
            rcvData[i] = p_evt->params.rx_data.p_data[i];//BRN					  
				  }
					newBleDataFlag = 1;
					
				}


    }

}
/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
 uint32_t           err_code;
    ble_nus_init_t     nus_init;
    nrf_ble_qwr_init_t qwr_init = {0};
		   ble_dfu_buttonless_init_t dfus_init = {0};

    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);

    // Initialize NUS.
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
		
		    // Initialize the async SVCI interface to bootloader.
    err_code = ble_dfu_buttonless_async_svci_init();
    APP_ERROR_CHECK(err_code);

    dfus_init.evt_handler = ble_dfu_evt_handler;

    err_code = ble_dfu_buttonless_init(&dfus_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
			  ledState = LED_ST_SLOW_BLINK;
        APP_ERROR_CHECK(err_code);
			  connectedFlag = 0; //BRN
				sysAutoOffCnt = AUTO_OFF_TIME;
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
	uint32_t err_code;

    nrf_gpio_pin_clear(LED_PIN);
	  nrf_gpio_pin_clear(HALL_PWR_PIN);
	  APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, 0));
	  app_pwm_disable(&PWM1); 
		app_timer_stop(m_app_1ms_tmr);
	  m_demo1_seq_values.channel_0 = 255;
	  nrf_drv_pwm_stop(&m_pwm0, true);
    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);
    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            //err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            //APP_ERROR_CHECK(err_code);
            break;
        case BLE_ADV_EVT_IDLE:
            //sleep_mode_enter();
            break;
        default:
            break;
    }
}





/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected");
            //err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            //APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
            APP_ERROR_CHECK(err_code);
            connectedFlag = 1;		
            beepTimes = 1;
            uploadYpState = 1;
            uploadYpStateDelay = 100;	
            aerateStartFlag = 0;
				    //advStartedFlag = 0;
            ledState = LED_ST_ON;					
				    //BRN
		        ble_gap_sec_params_t sec_param;
		        ble_gap_sec_params_t params;
            params.bond = 1;
            params.mitm = 1;
            sd_ble_gap_authenticate(m_conn_handle,&params);
				    NRF_LOG_INFO("authenticated");
				    NRF_LOG_FLUSH();
				    //BRN

				
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected");
            // LED indication will be changed when advertising starts.
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
				    connectedFlag = 0x00;//BRN
				    sysAutoOffCnt = AUTO_OFF_TIME;
				    beepTimes = 2;
				    ledState = LED_ST_SLOW_BLINK;
				    //advertising_init(); //BRN 
            break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;

//        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
//					  //BRN
//            // Pairing not supported
//            //err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
//            //APP_ERROR_CHECK(err_code);
//				    //resp_pair_request();
//				    //BRN
//            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
				    connectedFlag = 0; //BRN
				    sysAutoOffCnt = AUTO_OFF_TIME;
				    ledState = LED_ST_SLOW_BLINK;
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
				    ledState = LED_ST_SLOW_BLINK;
						connectedFlag = 0; //BRN
				    sysAutoOffCnt = AUTO_OFF_TIME;
            break;
//BRN
				 case BLE_GAP_EVT_AUTH_STATUS:
          if(p_ble_evt->evt.gap_evt.params.auth_status.auth_status == BLE_GAP_SEC_STATUS_SUCCESS){
            NRF_LOG_INFO("SEC_SUCCESS");
				    NRF_LOG_FLUSH();
						//beepTimes = 3;
          }else{
            sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
						ledState = LED_ST_SLOW_BLINK;
						connectedFlag = 0; //BRN
				    sysAutoOffCnt = AUTO_OFF_TIME;
          }
					//BRN
				
				
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for the SoftDevice initialization.
 *
 * @details This function initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}



static void peer_manager_init()
{
    ble_gap_sec_params_t sec_param;
    ret_code_t           err_code;

    err_code = pm_init();
    APP_ERROR_CHECK(err_code);

    memset(&sec_param, 0, sizeof(ble_gap_sec_params_t));

    // Security parameters to be used for all security procedures.
    sec_param.bond           = SEC_PARAM_BOND;
    sec_param.mitm           = SEC_PARAM_MITM;
    sec_param.lesc           = SEC_PARAM_LESC;
    sec_param.keypress       = SEC_PARAM_KEYPRESS;
    sec_param.io_caps        = SEC_PARAM_IO_CAPABILITIES;
    sec_param.oob            = SEC_PARAM_OOB;
    sec_param.min_key_size   = SEC_PARAM_MIN_KEY_SIZE;
    sec_param.max_key_size   = SEC_PARAM_MAX_KEY_SIZE;
    sec_param.kdist_own.enc  = 1;
    sec_param.kdist_own.id   = 1;
    sec_param.kdist_peer.enc = 1;
    sec_param.kdist_peer.id  = 1;

    err_code = pm_sec_params_set(&sec_param);
    APP_ERROR_CHECK(err_code);

    err_code = pm_register(pm_evt_handler);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**@brief Function for initializing the GATT library. */
void gatt_init(void)
{
    ret_code_t err_code;

    err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;
    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break;

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
				    connectedFlag = 0x00;//BRN
				    sysAutoOffCnt = AUTO_OFF_TIME;
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
						//beepTimes = 2;
            break;

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break;
				case BSP_EVENT_KEY_3:  //BRN
					   NRF_LOG_INFO("KEY3 EVENT HAPPENED");
				     //bsp_board_led_invert(BSP_BOARD_LED_1); //BRN 
					   DataLength = 4;
				     sendData[0] = 0xA1;
				     sendData[1] = 0xB2;
				     sendData[3] = 0xC3;
				     sendData[4] = testData++;
					   err_code = ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.tx_handles.value_handle );
             if ((err_code != NRF_ERROR_INVALID_STATE) &&
                            (err_code != NRF_ERROR_RESOURCES) &&
                            (err_code != NRF_ERROR_NOT_FOUND))
             {
               APP_ERROR_CHECK(err_code);
             }
        default:
            break;
				
				
    }
}





/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    uint32_t               err_code;
    ble_advertising_init_t init;
    int8_t        tx_power_level = TX_POWER_LEVEL;//BRN
    memset(&init, 0, sizeof(init));
	
	//--------------brn----------------------
	  //Զûݣ
	  uint8_t  my_adv_manuf_data[3];
//	  my_adv_manuf_data[0] = (uint8_t)(NRF_FICR->DEVICEID[0]&0x000000FF);
//	  my_adv_manuf_data[1] = (uint8_t)((NRF_FICR->DEVICEID[0] >> 8)&0x000000FF);
//	  my_adv_manuf_data[2] = (uint8_t)((NRF_FICR->DEVICEID[0] >> 16)&0x000000FF);
//	  my_adv_manuf_data[3] = (uint8_t)((NRF_FICR->DEVICEID[0] >> 24)&0x000000FF);
//		my_adv_manuf_data[4] = (uint8_t)(NRF_FICR->DEVICEID[1]&0x000000FF);
//	  my_adv_manuf_data[5] = (uint8_t)((NRF_FICR->DEVICEID[1] >> 8)&0x000000FF);
//	  my_adv_manuf_data[6] = (uint8_t)((NRF_FICR->DEVICEID[1] >> 16)&0x000000FF);
//	  my_adv_manuf_data[7] = (uint8_t)((NRF_FICR->DEVICEID[1] >> 24)&0x000000FF);
	  
	  my_adv_manuf_data[0] = 0;
    my_adv_manuf_data[1] = firmwareVer[0];
  	my_adv_manuf_data[2] = firmwareVer[1];
		
		//һԶݵĽṹù㲥ʱñĵֵַ
		//㲥ݰ
	  ble_advdata_manuf_data_t manuf_specific_data;
    //0x0059NordicIDID
		manuf_specific_data.company_identifier = 0x0059;
		//ָԶ
	  manuf_specific_data.data.p_data = my_adv_manuf_data;
		//ԶݴС
    manuf_specific_data.data.size   = sizeof(my_adv_manuf_data);
		
		init.advdata.p_manuf_specific_data    = &manuf_specific_data;
	//----------------------------------------------------------------------
	
	

    init.advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    init.advdata.include_appearance = false;
    init.advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;
    init.advdata.p_tx_power_level        = &tx_power_level; //BRN
    init.srdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    init.srdata.uuids_complete.p_uuids  = m_adv_uuids;

    init.config.ble_adv_fast_enabled  = true;
    init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
    init.config.ble_adv_fast_timeout  = APP_ADV_DURATION;
    init.evt_handler = on_adv_evt;

    err_code = ble_advertising_init(&m_advertising, &init);
    APP_ERROR_CHECK(err_code);

    ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}


/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(bool * p_erase_bonds)
{
    bsp_event_t startup_event;

    uint32_t err_code = bsp_init(BSP_INIT_LEDS | BSP_INIT_BUTTONS, bsp_event_handler);
    APP_ERROR_CHECK(err_code);

    err_code = bsp_btn_ble_init(NULL, &startup_event);
    APP_ERROR_CHECK(err_code);

    *p_erase_bonds = (startup_event == BSP_EVENT_CLEAR_BONDING_DATA);
}


/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    if (NRF_LOG_PROCESS() == false)
    {
        nrf_pwr_mgmt_run();
    }
}

/** @brief Clear bonding information from persistent storage.
 */
static void delete_bonds(void)
{
    ret_code_t err_code;

    NRF_LOG_INFO("Erase bonds!");

    err_code = pm_peers_delete();
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for starting advertising.
 */
/**@brief Function for starting advertising.
 */
static void advertising_start(bool erase_bonds)
{
      if (erase_bonds == true)
    {
        delete_bonds();
        // Advertising is started by PM_EVT_PEERS_DELETE_SUCCEEDED event.
    }
    else
    {
        uint32_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
        APP_ERROR_CHECK(err_code);

        NRF_LOG_DEBUG("advertising is started");
    }
	
//	uint32_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
//    APP_ERROR_CHECK(err_code);
}

static void tx_power_set(void)
{
    ret_code_t err_code = sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_ADV, m_advertising.adv_handle, TX_POWER_LEVEL);
    APP_ERROR_CHECK(err_code);
}




static void app_1ms_timer_handler(void * p_context)  //BRN
{
	  
    UNUSED_PARAMETER(p_context);
	
	  UNUSED_PARAMETER(p_context);
	  if(ledBlinkingCnt) ledBlinkingCnt--;
	  if(beepTimeCnt) beepTimeCnt--;
	  if(enterSleepDelay) enterSleepDelay--;
	  keyDebounceCnt++;
	  ypSwitchDetCnt++;
	  if(beepIntervalCnt) beepIntervalCnt--;
    if(uploadYpStateDelay) uploadYpStateDelay--;
	  if(adcReadDelay) adcReadDelay--;
	  hallSnrDebounceCnt++;
		rotateSpeedTimeCnt++;
    stuckCnt++;	
	  usbConnectCnt++;
	  if(motorOffDelayCnt) motorOffDelayCnt--;
	  if(hallBlockCnt) hallBlockCnt--;
	
		msCnt++;
		if(msCnt >= 1000)
		{
			msCnt = 0;
			if(sysAutoOffCnt) sysAutoOffCnt--;
			if((ypSwitchState == 0) && motorOnFlag)
			{
				if(aerateTimer) aerateTimer--;				
			}
		}
		
	  updateDevStateCnt++;
	  if(updateDevStateCnt >= 800)
		{
			updateDevStateCnt = 0;
				DataLength = 4;
		    sendData[0] = (uint8_t)(rotateSpeedTime >> 8);
		    sendData[1] = (uint8_t)(rotateSpeedTime & 0x00FF);
		    sendData[2] = m_demo1_seq_values.channel_0;
		    sendData[3] = testData++;
			  sendData[3] &= 0x7F;
			  if(backToFixedPosFlag) sendData[3] |= 0x80;
        ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.tx_handles.value_handle );	
	  }

}


static void GPIO_Init(void)
{
	nrf_gpio_cfg_output(LED_PIN);
	nrf_gpio_cfg_output(HALL_PWR_PIN);		
	nrf_gpio_pin_clear(LED_PIN);
	nrf_gpio_pin_clear(HALL_PWR_PIN); 

	nrf_gpio_cfg_input(KEY_PIN, NRF_GPIO_PIN_PULLUP );	
	nrf_gpio_cfg_input(YP_PIN, NRF_GPIO_PIN_PULLUP );	
	nrf_gpio_cfg_input(USB_DET_PIN, NRF_GPIO_PIN_PULLDOWN );
	nrf_gpio_cfg_input(HALL_OUT_PIN, NRF_GPIO_PIN_PULLUP );	
	nrf_gpio_cfg_input(BAT_VOL_PIN, NRF_GPIO_PIN_NOPULL );	
		
}

static void Button_Process(void)
{
	
	if((nrf_gpio_pin_read(KEY_PIN)) != preKeyState)
	{
		if(keyDebounceCnt >= 15)
		{
			preKeyState = nrf_gpio_pin_read(KEY_PIN);
			keyDebounceCnt = 0;
			if(preKeyState == 0) //KEY_DOWN
			{
				beepTimes = 1;
			  if(systemOnFlag == 0)
				{
					systemOnFlag = 1;
					sysAutoOffCnt = AUTO_OFF_TIME;
					ypSwitchState = 1; 
					ypSwitchDetCnt = 0;
					aerateTimer = 300;
					ledState = LED_ST_SLOW_BLINK;
					enterSleepDelay = 300;
					motorOnFlag = 0;
					backToFixedPosFlag = 0;
				}
				else
				{
					systemOnFlag = 0;
					backToFixedPosFlag = 0;
					if(motorOnFlag)
					{
					  motorDelayStopFlag = 1;
			      motorOffDelayCnt = 2000;
			      motorDelayOffTimeSetFlag = 0;
			      backToFixedPosFlag = 0;
					}
				}				
			}
		}
	}
	else
	{
		keyDebounceCnt = 0;
	}
}

static void LED_Process(void)
{
	if(systemOnFlag)
	{		
		if(batLowFlag)
		{
			if(ledBlinkingCnt == 0)
			{				
				if(nrf_gpio_pin_out_read(LED_PIN) == 1)
				{
					ledBlinkingCnt = 3000;
				}
				else
				{
					ledBlinkingCnt = 300;
					beepTimes = 1;
				}
				nrf_gpio_pin_toggle(LED_PIN);
			}				
		}
		else if(ledState == 0)
		{
			nrf_gpio_pin_clear(LED_PIN);
		}
		else if(ledState == 1) //SLOW BLINKING
		{
			if(ledBlinkingCnt == 0)
			{
				ledBlinkingCnt = 1000;
				nrf_gpio_pin_toggle(LED_PIN);
			}	
		}
		else if(ledState == 2) //constantly on
		{
			nrf_gpio_pin_set(LED_PIN);
		}
		else if(ledState == 3) //FAST BLINKING
		{
			if(ledBlinkingCnt == 0)
			{
				
				if(nrf_gpio_pin_out_read(LED_PIN) == 1)
				{
					ledBlinkingCnt = 300;
				}
				else
				{
					ledBlinkingCnt = 100;
				}
				nrf_gpio_pin_toggle(LED_PIN);
			}	
		}				
	}
	else
	{
		nrf_gpio_pin_clear(LED_PIN);		
	}
	
	
}

static void Motro_Ctrl(void)
{
	
	if(stuckedFlag)
	{
		if(motorOnFlag)
		{			
			m_demo1_seq_values.channel_0 = 255;
			motorOnFlag = 0;
		}
	}
	
	if((ypSwitchState == 1) && backToFixedPosFlag)
	{
		if(motorDelayStopFlag == 0)
		{			
		  pwmSetting = INIT_BAT_PWM;
		  pwmSettingBackup = pwmSetting;
			m_demo1_seq_values.channel_0 = 255 - pwmSetting;
		  motorOnFlag = 1;
		  stuckCnt = 0;
			motorDelayStopFlag = 1;
			motorOffDelayCnt = 2000;
			hallBlockCnt = 30;
			motorDelayOffTimeSetFlag = 0;
			backToFixedPosFlag = 0;
		}
	}
	
	if(motorDelayStopFlag && motorOnFlag)
	{
		if(motorOffDelayCnt == 0)
		{
			m_demo1_seq_values.channel_0 = 255;
			motorOnFlag = 0;
			motorDelayStopFlag = 0;
		}		
	}
	
	if(systemOnFlag && aerateTimer && (ypSwitchState == 0) && aerateStartFlag && (stuckedFlag == 0))
	{
		if(motorOnFlag == 0)
		{
			pwmSetting = INIT_BAT_PWM;
			pwmSettingBackup = pwmSetting;
			m_demo1_seq_values.channel_0 = 255 - pwmSetting;
			motorOnFlag = 1;
			stuckCnt = 0;
			motorDelayStopFlag = 0;
			backToFixedPosFlag = 1;
		}		
	}
	else if(aerateDoneFlag) 
	{
		if(backToFixedPosFlag)
		{
			motorDelayStopFlag = 1;
			motorOffDelayCnt = 2000;
			motorDelayOffTimeSetFlag = 0;
			backToFixedPosFlag = 0;
			m_demo1_seq_values.channel_0 = 255 - pwmSetting;
			motorOnFlag = 1;
		}	
		else if(motorOnFlag && (motorDelayStopFlag == 0))
		{
			m_demo1_seq_values.channel_0 = 255;
			motorOnFlag = 0;
		}		
	}else if(motorDelayStopFlag == 0)
	{
		if(motorOnFlag)
		{
			m_demo1_seq_values.channel_0 = 255;
			motorOnFlag = 0;
		}
	}
	
}

static void Beep_Ctrl(void)
{
	if(aerateDoneBeepCnt)
	{
		if(beepIntervalCnt == 0)
		{
			beepTimes = 3;
			beepIntervalCnt = 3000;
			aerateDoneBeepCnt--;
		}		
	}
	
	if(stuckedFlag)
	{
		if(beepIntervalCnt == 0)
		{
			beepTimes = 1;
			beepIntervalCnt = 500;
		}		
	}
	
	
	if(beepTimeCnt == 0)
	{
		if(beepTimes)
		{
			if(beepOnFlag)
			{
				beepOnFlag = 0;
				APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, 0));
				beepTimeCnt = 200;
				beepTimes--;
			}
			else
			{
				beepOnFlag = 1;
				APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, 50));
				beepTimeCnt = 100;
			}			
		}
	}
}


void YP_Swtich_Detection(void)
{
	
	if(uploadYpState && (uploadYpStateDelay == 0))
	{
		uploadYpState = 0;
		DataLength = 1;
		if(ypSwitchState == 0)
		{
		  sendData[0] = 0x01;
		}
		else
		{
			sendData[0] = 0x00;
		}
		ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.pot_handles.value_handle );		
	}
	
	if(nrf_gpio_pin_read(YP_PIN) != ypSwitchState)
	{
		if(ypSwitchDetCnt >= 300)
		{
			ypSwitchState = nrf_gpio_pin_read(YP_PIN);
			ypSwitchDetCnt = 0;
			if(ypSwitchState == 1) //YP SWITCH is up
			{
				stuckedFlag = 0;
				if(aerateDoneFlag == 0)
				{
				  sysAutoOffCnt = AUTO_OFF_TIME;
				}				
				DataLength = 1;
				sendData[0] = 0x00;
				ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.pot_handles.value_handle );
			}
			else
			{
				//aerateTimer = 300;
				//ledState = LED_ST_ON;	
        				
				DataLength = 1;
				sendData[0] = 0x01;
				ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.pot_handles.value_handle );
				if(connectedFlag == 0)
				{
					if((aerateStartFlag == 0) && (ledState != LED_ST_FAST_BLINK))
					{
					  //aerateTimer = 300;
					  //aerateStartFlag = 1;
					}
				}
			}			
		}
	}
	else
	{
		ypSwitchDetCnt = 0;
	}
}

static void Timer_Process(void)
{
	if(aerateDoneFlag)
	{
		if(sysAutoOffCnt == 0)
		{
			systemOnFlag = 0;	
			enterSleepDelay = 300;
			beepTimes = 1;
			aerateDoneFlag = 0;
		}		
	}
	
	
	if(stuckedFlag || batLowFlag)
	{
		if(systemOnFlag &&  (sysAutoOffCnt == 0))
		{
			systemOnFlag = 0;	
			enterSleepDelay = 500;
			beepTimes = 1;
		}
	}
	
	if((ypSwitchState == 1) || (aerateStartFlag == 0))
	{
		if(systemOnFlag && (sysAutoOffCnt == 0) && (connectedFlag == 0)){
			systemOnFlag = 0;	
			enterSleepDelay = 500;
			beepTimes = 1;
		}
	}
	else if( (aerateStartFlag) && (aerateTimer == 0))
	{
		ledState = LED_ST_FAST_BLINK;
		sysAutoOffCnt = 10;
		aerateDoneBeepCnt = 3;
		aerateStartFlag = 0;
		aerateDoneFlag = 1;
	}	
}


void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
//   
}

void saadc_init(void)
{
    ret_code_t err_code;
    nrf_saadc_channel_config_t channel_config =
            NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN6);
    err_code = nrf_drv_saadc_init(NULL, saadc_callback);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_channel_init(0, &channel_config);
    APP_ERROR_CHECK(err_code);

}

void Adc_Read(void)
{
	if(adcReadDelay == 0)
	{
		adcReadDelay = 500;
		nrf_drv_saadc_sample_convert(0,&saadc_val);
		if((batLowFlag == 0) && (usbConnectedFlag == 0))
		{
			if(saadc_val <= 0x02C8)
			{
				batLowCnt++;
				if(batLowCnt >= 3)
				{
					batLowFlag = 1;
					if(connectedFlag)
					{
						sysAutoOffCnt = AUTO_OFF_TIME;
					}
				}				
			}else
			{
				batLowCnt = 0;
			}
		}
//		DataLength = 4;
//		sendData[0] = (uint8_t)(saadc_val >> 8);
//		sendData[1] = (uint8_t)(saadc_val & 0x00FF);
//		sendData[2] = 0xC3;
//		sendData[3] = testData++;
//    ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.tx_handles.value_handle );	
	}
	
}

void Usb_Detect(void)
{
	if(nrf_gpio_pin_read(USB_DET_PIN) != usbConnectedFlag)
	{
		if(usbConnectCnt > 100)
		{
			usbConnectedFlag = nrf_gpio_pin_read(USB_DET_PIN);
			if(usbConnectedFlag)
			{
				beepTimes = 1;
				batLowFlag = 0;
			}
			else
			{
				beepTimes = 2;
			}
		}		
	}
	else
	{
		usbConnectCnt = 0;
	}
}

void Stuck_Detect(void)
{
	if(motorOnFlag && (ypSwitchState == 0))
	{
		if(stuckCnt > 2500)
		{
			stuckedFlag = 1;
			stuckCnt = 0;
			sysAutoOffCnt = AUTO_OFF_TIME;
		}		
	}	
}




void Hall_Sensor_Detect(void)
{
  if(nrf_gpio_pin_read(HALL_OUT_PIN) != preHallSnrLvl)
	{
		if(hallSnrDebounceCnt >= 3)
		{
			preHallSnrLvl = nrf_gpio_pin_read(HALL_OUT_PIN);
			hallSnrDebounceCnt = 0;
			stuckCnt = 0;
			hallStUpdateFlag = 1;
			if(preHallSnrLvl == 0)
			{
				rotateSpeedTime = rotateSpeedTimeCnt;
				rotateSpeedTimeCnt = 0;
//				DataLength = 4;
//		    sendData[0] = (uint8_t)(rotateSpeedTime >> 8);
//		    sendData[1] = (uint8_t)(rotateSpeedTime & 0x00FF);
//		    sendData[2] = pwmSetting;
//		    sendData[3] = testData++;
//        ble_nus_data_send(&m_nus, sendData, &DataLength, m_conn_handle, m_nus.tx_handles.value_handle );	
				
				if(motorDelayStopFlag && (motorDelayOffTimeSetFlag == 0) && (hallBlockCnt == 0))
				{
					motorOffDelayCnt = motorDelayOffTimeSet;
					motorDelayOffTimeSetFlag = 1;
				}				
			}			
		}		
	}
	else
	{
		hallSnrDebounceCnt = 0;
	}	
}

void Rotate_Speed_Adjust(void)
{
	if(aerateStartFlag && motorOnFlag && hallStUpdateFlag)
	{
		hallStUpdateFlag = 0;
		if(rotateSpeedTime >= 933)
		{
			if(pwmSetting < 255)
			{
				pwmSetting++;
				pwmSettingBackup = pwmSetting;
				m_demo1_seq_values.channel_0 = 255 - pwmSetting;
			}
			
		}
		else if(rotateSpeedTime <= 913)
		{
			if(pwmSetting)
			{
				pwmSetting--;
				pwmSettingBackup = pwmSetting;
				m_demo1_seq_values.channel_0 = 255 - pwmSetting;
			}			
		}
	}
	
}



void variable_init(void)
{
  preKeyState = 1;
	ypSwitchState = 1;
	enterSleepDelay = 500;
	aerateStartFlag = 0;
	ledState = LED_ST_SLOW_BLINK;
	uploadYpState = 0;
	adcReadDelay = 100;
	batLowFlag = 0;
}




/**@brief Application main function.
 */
int main(void)
{
    bool erase_bonds;
	  uint32_t err_code;

	  GPIO_Init();
	  //beepTimes = 2; //test
	  if(nrf_gpio_pin_read(KEY_PIN) == 0)
		{
			beepTimes = 1;
		  systemOnFlag = 1;
			sysAutoOffCnt = AUTO_OFF_TIME;//60;
			ypSwitchState = 1; 
			ypSwitchDetCnt = 0;
			aerateTimer = 0;
			ledState = LED_ST_SLOW_BLINK;
			enterSleepDelay = 300;
			motorOnFlag = 0;
			aerateStartFlag = 0;
			aerateDoneFlag = 0;
			batLowFlag = 0;
			nrf_gpio_pin_set(HALL_PWR_PIN);
			nrf_gpio_pin_set(LED_PIN);
			if(nrf_gpio_pin_read(USB_DET_PIN))
			{
				usbConnectedFlag = 1;
			}
			else
			{
				usbConnectedFlag = 0;
			}
			motorDelayOffTimeSet = 500;
		}
		
		pwm_drv_init();
	      /* Initialize and enable PWM. */
		app_pwm_config_t pwm1_cfg = APP_PWM_DEFAULT_CONFIG_1CH(250, BEEP_PIN); //BEEP /MOTOR
    pwm1_cfg.pin_polarity[0] = APP_PWM_POLARITY_ACTIVE_HIGH;
		
    brnEc = app_pwm_init(&PWM1,&pwm1_cfg,NULL/*pwm_ready_callback*/);
    APP_ERROR_CHECK(brnEc);
    app_pwm_enable(&PWM1);
	  APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, 0));
				
		    
		while(beepTimes)
		{
			APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, 50));
			nrf_delay_ms(50);
			APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, 0));
			nrf_delay_ms(100);
			beepTimes--;
		}
		
    // Initialize.
    log_init();
    timers_init();
    //buttons_leds_init(&erase_bonds);
	  //bsp_board_init(BSP_INIT_LEDS); //BRN
    power_management_init();
    ble_stack_init();
	  peer_manager_init();
    gap_params_init();
    gatt_init();
    services_init();
    advertising_init();
    conn_params_init();
	  variable_init();
    // Start execution.
    //advertising_start(erase_bonds);
    NRF_LOG_INFO("Debug logging for UART over RTT started.");
    app_timer_create(&m_app_1ms_tmr,APP_TIMER_MODE_REPEATED,app_1ms_timer_handler); //BRN	
    app_timer_start(m_app_1ms_tmr, APP_TIMER_TICKS(1), NULL);   

   	saadc_init();
		

    // Enter main loop.
    for (;;)
    {
			
      Button_Process();
			LED_Process();
			Beep_Ctrl();
			YP_Swtich_Detection();
			Motro_Ctrl();
			Timer_Process();
			Adc_Read();
			Hall_Sensor_Detect();
			Rotate_Speed_Adjust();
			Usb_Detect();
			Stuck_Detect();
			
			if(newBleDataFlag)
			{
				newBleDataFlag = 0;
				beepTimes = 1;
				APP_ERROR_CHECK(app_pwm_channel_duty_set(&PWM1, 0, rcvData[3]));
				//m_demo1_seq_values.channel_0 = rcvData[3];
				if(ypSwitchState == 1)
				{
					//beepTimes = 2;
				}
				else if(aerateDoneFlag == 0)
				{
					beepTimes = 1;
					//motorDelayOffTimeSet = (rcvData[2] * 256) + rcvData[3];  //test
					
					if(rcvData[0] == 0x01)
					{
					  aerateTimer = rcvData[1]*60 + rcvData[2];
					  aerateStartFlag = 1;
						aerateDoneFlag = 0;
					}
					else
					{
						aerateStartFlag = 0;
						aerateDoneFlag = 0;
						if(motorOnFlag)
					  {
					    motorDelayStopFlag = 1;
			        motorOffDelayCnt = 2000;
			        motorDelayOffTimeSetFlag = 0;
			        backToFixedPosFlag = 0;
					  }
					}
				}				
			}
			
			
			if((enterSleepDelay == 0) &&(systemOnFlag == 0) && (nrf_gpio_pin_read(KEY_PIN) == 1))
			{  								
				if((advStartedFlag) && (connectedFlag == 0))
				{
					advStartedFlag = 0;
//				  err_code = sd_ble_gap_adv_stop(m_advertising.adv_handle);
//				  APP_ERROR_CHECK(err_code);
				}
        if(motorDelayStopFlag == 0)
				{
			    sleep_mode_enter();
				}
      }
      else
			{
        if(systemOnFlag && (advStartedFlag == 0))
				{
					advertising_start(false);
					advStartedFlag = 1;
				}

			}										
			 idle_state_handle();
    }
}


/**
 * @}
 */
